@@ -1,3 +1,4 @@
+# pylint: disable=ungrouped-imports
 import hypothesis_jsonschema._from_schema
 import jsonschema
 import werkzeug
@@ -20,6 +21,12 @@ else:
         pass
 
 
+try:
+    from hypothesis.utils.conventions import InferType
+except ImportError:
+    InferType = type(...)
+
+
 def _get_format_filter(
     format_name: str,
     checker: jsonschema.FormatChecker,
