@@ -3,19 +3,18 @@ from queue import Queue
 from . import events
 from .client import ServiceClient
 from .constants import STOP_MARKER
+from .models import TestRun
 from .serialization import serialize_event
 
 
-def start(url: str, token: str, in_queue: Queue, out_queue: Queue) -> None:
+def start(client: ServiceClient, test_run: TestRun, in_queue: Queue, out_queue: Queue) -> None:
     """Initialize a new run and start consuming events."""
     try:
-        client = ServiceClient(url, token)
-        response = client.create_test_run()
-        consume_events(client, in_queue, response.run_id)
+        consume_events(client, in_queue, test_run.run_id)
         # Reached a terminal event or a stop marker.
         # In the case of stop marker, it is still a successful result for the handler itself as the error happened in
         # a different handler
-        out_queue.put(events.Completed(short_url=response.short_url))
+        out_queue.put(events.Completed(short_url=test_run.short_url))
     except Exception as exc:
         out_queue.put(events.Error(exc))
 
