@@ -1,6 +1,7 @@
-import functools
 import json
-from typing import Any, Callable, Dict, Generator, List, Optional
+from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, Union
+
+from ...utils import compose
 
 Generated = Dict[str, Any]
 Definition = Dict[str, Any]
@@ -8,15 +9,6 @@ DefinitionList = List[Definition]
 MapFunction = Callable[[Generated], Generated]
 
 
-def compose(*functions: Callable) -> Callable:
-    """Compose multiple functions into a single one."""
-
-    def noop(x: Any) -> Any:
-        return x
-
-    return functools.reduce(lambda f, g: lambda x: f(g(x)), functions, noop)
-
-
 def make_serializer(
     func: Callable[[DefinitionList], Generator[Optional[Callable], None, None]]
 ) -> Callable[[DefinitionList], Optional[Callable]]:
@@ -165,7 +157,7 @@ serialize_swagger2_parameters = make_serializer(_serialize_swagger2)
 def conversion(func: Callable[..., None]) -> Callable:
     def _wrapper(name: str, **kwargs: Any) -> MapFunction:
         def _map(item: Generated) -> Generated:
-            if name in item:
+            if name in (item or {}):
                 func(item, name, **kwargs)
             return item
 
@@ -175,7 +167,27 @@ def conversion(func: Callable[..., None]) -> Callable:
 
 
 def make_delimited(data: Optional[Dict[str, Any]], delimiter: str = ",") -> str:
-    return delimiter.join(f"{key}={value}" for key, value in (data or {}).items())
+    return delimiter.join(f"{key}={value}" for key, value in force_dict(data or {}).items())
+
+
+def force_iterable(value: Any) -> Union[List, Tuple]:
+    """Converts the value to a list or a tuple.
+
+    Only relevant for negative test scenarios where the original types might be changed.
+    """
+    if isinstance(value, (tuple, list)):
+        return value
+    return [value]
+
+
+def force_dict(value: Any) -> Dict:
+    """Converts the value to a dictionary.
+
+    Only relevant for negative test scenarios where the original types might be changed.
+    """
+    if isinstance(value, dict):
+        return value
+    return {"": value}
 
 
 @conversion
@@ -186,7 +198,7 @@ def to_json(item: Generated, name: str) -> None:
 
 @conversion
 def delimited(item: Generated, name: str, delimiter: str) -> None:
-    item[name] = delimiter.join(map(str, item[name] or ()))
+    item[name] = delimiter.join(map(str, force_iterable(item[name] or ())))
 
 
 @conversion
@@ -197,14 +209,14 @@ def deep_object(item: Generated, name: str) -> None:
     """
     generated = item.pop(name)
     if generated:
-        item.update({f"{name}[{key}]": value for key, value in generated.items()})
+        item.update({f"{name}[{key}]": value for key, value in force_dict(generated).items()})
     else:
         item[name] = ""
 
 
 @conversion
 def comma_delimited_object(item: Generated, name: str) -> None:
-    item[name] = ",".join(map(str, sum((item[name] or {}).items(), ())))
+    item[name] = ",".join(map(str, sum((force_dict(item[name] or {})).items(), ())))
 
 
 @conversion
@@ -251,7 +263,7 @@ def label_array(item: Generated, name: str, explode: Optional[bool]) -> None:
         delimiter = "."
     else:
         delimiter = ","
-    new = delimiter.join(map(str, item[name] or ()))
+    new = delimiter.join(map(str, force_iterable(item[name] or ())))
     if new:
         item[name] = f".{new}"
     else:
@@ -273,7 +285,7 @@ def label_object(item: Generated, name: str, explode: Optional[bool]) -> None:
     if explode:
         new = make_delimited(item[name], ".")
     else:
-        object_items = map(str, sum((item[name] or {}).items(), ()))
+        object_items = map(str, sum(force_dict(item[name] or {}).items(), ()))
         new = ",".join(object_items)
     if new:
         item[name] = f".{new}"
@@ -307,9 +319,9 @@ def matrix_array(item: Generated, name: str, explode: Optional[bool]) -> None:
         id=[3, 4, 5] => ";id=3,4,5"
     """
     if explode:
-        new = ";".join(f"{name}={value}" for value in item[name] or ())
+        new = ";".join(f"{name}={value}" for value in force_iterable(item[name] or ()))
     else:
-        new = ",".join(map(str, item[name] or ()))
+        new = ",".join(map(str, force_iterable(item[name] or ())))
     if new:
         item[name] = f";{new}"
     else:
@@ -331,7 +343,7 @@ def matrix_object(item: Generated, name: str, explode: Optional[bool]) -> None:
     if explode:
         new = make_delimited(item[name], ";")
     else:
-        object_items = map(str, sum((item[name] or {}).items(), ()))
+        object_items = map(str, sum(force_dict(item[name] or {}).items(), ()))
         new = ",".join(object_items)
     if new:
         item[name] = f";{new}"
